@@ -6,9 +6,9 @@ import numpy
 import pystare
 import shapely
 import geopandas
+from shapely.geometry import Point
 
-
-def to_vertices(sids, wrap_lon=True, wrap_thresh=180):
+def to_vertices(sids, wrap_lon=True):
     """ Converts a (collection of) sid(s) into vertices. Vertices are a tuple of:
 
     1. the latitudes of the corners
@@ -25,8 +25,6 @@ def to_vertices(sids, wrap_lon=True, wrap_thresh=180):
     wrap_lon: bool
         toggle if vertices should be wraped around antimeridian. If true:
         lon = ((lon + 180.0) % 360.0) - 180.0
-    wrap_thresh: float
-        the longitude around which we should wrap.
 
     Returns
     ---------
@@ -45,10 +43,12 @@ def to_vertices(sids, wrap_lon=True, wrap_thresh=180):
      array([15.84277554]))
     """
     vs = pystare.to_vertices_latlon(sids)
+
     if wrap_lon:
-        # We only wrap longitudes over wrap_thresh degrees.
-        vs[1][vs[1] > wrap_thresh] = (vs[1][vs[1] > wrap_thresh] + 180) % 360.0 - 180
-        vs[3][vs[3] > wrap_thresh] = (vs[3][vs[3] > wrap_thresh] + 180) % 360.0 - 180
+        # If the trixel is not counterclockwise, then it is wrapped around the antimeridian
+        # and we need to move the vertices over
+        vs[1][vs[1]>180] = (vs[1][vs[1]>180] + 180) % 360.0 - 180
+        vs[3][vs[3]>180] = (vs[3][vs[3]>180] + 180) % 360.0 - 180
     return vs
 
 
@@ -258,7 +258,7 @@ def vertices2gring(vertices):
     grings: numpy.array
         The great circles constraining the trixels given by their ECEF norm vectors.
     """
-    corners = vertices2corners(vertices)
+    corners = vertices2corners_ecef(vertices)
     return corners2gring(corners)
 
 
@@ -333,8 +333,8 @@ def to_centerpoints(sids):
     >>> import starepandas
     >>> sids = [4611263805962321926, 4611404543450677254]
     >>> centerpoints = starepandas.to_centerpoints(sids)
-    >>> print(centerpoints[0])
-    POINT (19.50219017924583 23.29074702177385)
+    >>> print(Point(19.50219017924583,23.29074702177385).distance(centerpoints[0]) < 1.0e-12)
+    True
     """
 
     vertices = to_vertices(sids)
@@ -342,7 +342,7 @@ def to_centerpoints(sids):
     return centerpoints
 
 
-def to_corners(sids, wrap_lon=True, wrap_thresh=180):
+def to_corners(sids, wrap_lon=True):
     """ Converts a (collection of) sid(s) into (collection of) corners in lon/lat representation.
 
     Parameters
@@ -372,7 +372,7 @@ def to_corners(sids, wrap_lon=True, wrap_thresh=180):
             [19.73607532, 24.53819039]]])
     """
 
-    vertices = to_vertices(sids, wrap_lon=wrap_lon, wrap_thresh=wrap_thresh)
+    vertices = to_vertices(sids, wrap_lon=wrap_lon)
     corners = vertices2corners(vertices)
     return corners
 
@@ -451,7 +451,7 @@ def corners2gring(corners):
     return gcs
 
 
-def to_trixels(sids, as_multipolygon=False, wrap_lon=True, wrap_thresh=180):
+def to_trixels(sids, as_multipolygon=False, wrap_lon=True):
     """
     Converts a (collection of) sid(s) into a (collection of) trixel(s)
 
@@ -464,8 +464,6 @@ def to_trixels(sids, as_multipolygon=False, wrap_lon=True, wrap_thresh=180):
         combined into a multipolygon. Otherwise a list of trixels is returned.
     wrap_lon: bool
         toggle if trixels should be wraped around antimeridian.
-    wrap_thresh: float
-        longitude at which values should be wrapped
 
     Returns
     ---------
@@ -488,7 +486,7 @@ def to_trixels(sids, as_multipolygon=False, wrap_lon=True, wrap_thresh=180):
         sids = sids.astype(numpy.int64)
 
     trixels = []
-    vertices = to_corners(sids, wrap_lon=wrap_lon, wrap_thresh=wrap_thresh)
+    vertices = to_corners(sids, wrap_lon=wrap_lon)
     for vertex in vertices:
         geom = shapely.geometry.Polygon(vertex)
         trixels.append(geom)
@@ -500,7 +498,7 @@ def to_trixels(sids, as_multipolygon=False, wrap_lon=True, wrap_thresh=180):
     return trixels
 
 
-def trixels_from_stareseries(sids_series, n_workers=1, wrap_lon=True, wrap_thresh=180):
+def trixels_from_stareseries(sids_series, n_workers=1, wrap_lon=True):
     """ Takes a series of STARE index values and creates an array of sets of trixels. If a row contains a set of sids
     (rather than a single sid); i.e. representing e.g. a region, a set of trixels will be generated and combined in a
     multipolygon
@@ -526,25 +524,25 @@ def trixels_from_stareseries(sids_series, n_workers=1, wrap_lon=True, wrap_thres
     >>> sdf = starepandas.STAREDataFrame(sids=sids)
     >>> trixels = starepandas.trixels_from_stareseries(sdf.sids)
     """
+    npartitions = n_workers
     if len(sids_series) <= 1:
-        n_workers = 1
-    elif n_workers >= len(sids_series):
+        npartitions = 1
+    elif npartitions >= len(sids_series):
         # Cannot have more partitions than rows
-        n_workers = len(sids_series) - 1
+        npartitions = len(sids_series) - 1
 
-    if n_workers == 1:
+    if npartitions == 1:
         trixels_series = []
         for sids in sids_series:
-            trixels = to_trixels(sids, as_multipolygon=True, wrap_lon=wrap_lon, wrap_thresh=wrap_thresh)
+            trixels = to_trixels(sids, as_multipolygon=True, wrap_lon=wrap_lon)
             trixels_series.append(trixels)
     else:
-        ddf = dask.dataframe.from_pandas(sids_series, npartitions=n_workers)
+        ddf = dask.dataframe.from_pandas(sids_series, npartitions=npartitions)
         meta = {'trixels': 'object'}
         res = ddf.map_partitions(lambda df:
                                  vectorized.from_shapely(
                                      trixels_from_stareseries(df, n_workers=1,
-                                                              wrap_lon=wrap_lon,
-                                                              wrap_thresh=wrap_thresh)).flatten(),
+                                                              wrap_lon=wrap_lon)).flatten(),
                                  meta=meta)
         trixels_series = res.compute(scheduler='processes')
         # Since the array would be ragged, we are probably safer with a list of arrays
@@ -553,6 +551,41 @@ def trixels_from_stareseries(sids_series, n_workers=1, wrap_lon=True, wrap_thres
     return trixels_series
 
 
+def split_antimeridian_series(trixels_series, n_workers=1):
+    npartitions = n_workers
+
+    if len(trixels_series) <= 1:
+        npartitions = 1
+    elif npartitions >= len(trixels_series):
+        # Cannot have more partitions than rows
+        npartitions = len(trixels_series) - 1
+
+    if npartitions == 1:
+        split = []
+        for row in trixels_series:
+            if row.geom_type == 'Polygon':
+                # We need to catch single Polygons
+                row = [row]
+            row = split_antimeridian(row)
+            split.append(row)
+        trixels_series = geopandas.GeoSeries(split,
+                                             crs='EPSG:4326',
+                                             index=trixels_series.index)
+    else:
+        ddf = dask.dataframe.from_pandas(trixels_series, npartitions=npartitions)
+        meta = {'trixels': 'object'}
+        res = ddf.map_partitions(lambda df:
+                                 vectorized.from_shapely(split_antimeridian_series(df, n_workers=1)).flatten(),
+                                 meta=meta)
+        trixels_series = res.compute(scheduler='processes')
+        # Since the array would be ragged, we are probably safer with a list of arrays
+        trixels_series = trixels_series.tolist()
+    trixels_series = geopandas.GeoSeries(trixels_series,
+                                         crs='EPSG:4326',
+                                         index=trixels_series.index)
+    return trixels_series
+
+
 def split_antimeridian(trixels):
     """Splits trixels at the antimeridian
 
@@ -564,13 +597,21 @@ def split_antimeridian(trixels):
     trixels: A polygon, multipolygon, collection of polygons, or a geometry series
         A collection of trixels.
     """
-
     bbox = shapely.geometry.Polygon([(-180, -90), (180, -90), (180, 90), (-180, 90)])
 
     trixels = geopandas.GeoSeries(trixels, crs='EPSG:4326')
 
     exploded = trixels.explode(index_parts=True).reset_index(drop=True)
 
+    for idx, trixel in exploded.iteritems():
+        if not trixel.exterior.is_ccw:
+            # If trixels are not CCW they have been constructed incorrectly
+            # and we wrap their verices around the antimeridian
+            x = (numpy.array(trixel.exterior.xy[0]) + 180) % 360.0 - 180
+            y = numpy.array(trixel.exterior.xy[1])
+            # TODO: this appears to break
+            exploded[idx] = shapely.geometry.Polygon(zip(x, y))
+
     inside = exploded.intersection(bbox)
     inside[inside.geom_type != 'Polygon'] = shapely.wkt.loads('POLYGON EMPTY')
     inside = geopandas.tools.collect(inside, multi=True)
